//+------------------------------------------------------------------+
//|		                                                    NeuroNet.h |
//|					                         Copyright 2009,   |
//|                                              profy.mql@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2009,  "
#property link      "mailto: profy.mql@gmail.com"

#include <Object.mqh>
#include <Files\File.mqh>

#define RAND_MAX	32767
#define iMaxLayer	10
#define iMaxNeuron	100
#define iMaxPattern	10000

//   (LayerType) (   )
#define ltIn	0
#define ltOut	1
#define ltBack	2

//------------------------------------------------
class CNeuroNet : public CObject  
{
	int nBAR, nIN, nOUT;
public:

	int nCycle;		//     
	int nPattern;	//   
	int nLayer;		//   
	double Delta;	//    
	int nNeuron[iMaxLayer];			//     ( )
	int LayerType[iMaxLayer];		//   ( )
	double W[iMaxLayer][iMaxNeuron][iMaxNeuron];//   
	double dW[iMaxLayer][iMaxNeuron][iMaxNeuron];//  
	double Thresh[iMaxLayer][iMaxNeuron];		// 
	double dThresh[iMaxLayer][iMaxNeuron];		//  
	double Out[iMaxLayer][iMaxNeuron];			//  
	double Err[iMaxLayer][iMaxNeuron][iMaxNeuron];// 

	double Speed;			//  
	double Impuls;			//  

	double in[100][iMaxPattern];	//   
	double out[10][iMaxPattern];	//   
	double pout[10];				//    
	MqlRates bar[iMaxPattern];		// ,   
	int	TradePos;	//  
	double ProfitPos;	//  / 

public:
	CNeuroNet();
	virtual ~CNeuroNet();
	// 
	void Init(int aPattern=1, int aLayer=1, int aCycle=10000, double aDelta=0.01, double aSpeed=0.1, double aImpuls=0.1);
	//  
	void CalculateLayer();	//   
	void CalculateError();	//   /  Target/
	void ChangeWeight();	//  
	void Step(int pat);		//      
	void Walk();					//      
	bool Run();						//    
	bool CountError();		//   
	bool TrainMPS();	//      

	//      
	bool bInProc;	//     TrainNetwork
	bool bStop;		//      TrainNetwork
	int loop;
	int pat;		
	double iMaxErr;	//    
	double dMaxErr;	//  
	double sErr;	//   


	void LinearCovariation();	//  
	void Load(string );				//  
	void LoadHistory(string );				//  

	void LoadW(string );				//  
	void SaveW(string );				//  
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CNeuroNet::CNeuroNet()
{
	nCycle=10000; Delta=0.01; nPattern=1;
	sErr=0; loop=0; pat=0; iMaxErr=0; dMaxErr=0; bInProc=false; bStop=false;
}
//------------------------------------------------
CNeuroNet::~CNeuroNet()
{
}
//------------------------------------------------
void CNeuroNet::Init(int aPattern, int aLayer, int aCycle, double aDelta, double aSpeed, double aImpuls)
{
	//  
	nCycle=aCycle; Delta=aDelta; nPattern=aPattern; nLayer=aLayer; Speed=aSpeed; Impuls=aImpuls;
	if (nPattern>iMaxPattern)	{ MessageBox("   !"); nPattern=iMaxPattern; }
	//  
	int N, pN, L;
	string st;
	//   
	LayerType[0]=ltIn;	//   (  in)
	LayerType[nLayer]=ltOut; //   (  out)
	for(L=1;L<nLayer;L++) LayerType[L]=ltBack; //      

	srand(TimeCurrent()); //  
	double p=0.00001;
	//      
	for(L=1;L<nLayer;L++)
	{
		for(N=0;N<nNeuron[L];N++)	
		{
			for(pN=0;pN<nNeuron[L-1];pN++) 
			{
				dW[L][N][pN]=0; W[L][N][pN]=p+double(rand())/RAND_MAX;
				st=st+"L "+ITS(L)+"   N  "+ITS(N)+"   pN "+ITS(pN)+"  dW "+DTS(dW[L][N][pN])+"  W "+DTS(W[L][N][pN]);
			}
			dThresh[L][N]=0; Thresh[L][N]=p+double(rand())/RAND_MAX;
			st=st+"\n";
		}
	}

	//      [-1,1]
//	LinearCovariation();
}
//------------------------------------------------
void CNeuroNet::CalculateLayer() //   
{
	int N, pN, L;
	double sum;
	string st="";
	for(L=1;L<nLayer;L++) //   
	{
		if (LayerType[L]==ltBack) //  -   
		{
			for(N=0;N<nNeuron[L];N++)
			{ 
				sum=0;
				st=st+"L "+ITS(L)+"   N  "+ITS(N);
				for(pN=0;pN<nNeuron[L-1];pN++) 
				{
					sum+=W[L][N][pN]*Out[L-1][pN];
					st=st+"  W "+DTS(W[L][N][pN])+"  ppOut "+DTS(Out[L-1][pN]);
				}
				Out[L][N]=Sigmoid(sum+Thresh[L][N]);
				st=st+"   sum "+DTS(sum)+"  Out "+DTS(Out[L][N])+"\n";
			}
		}
	}
//	Comment(st);
}
//------------------------------------------------
void CNeuroNet::CalculateError() //  
{
	int N, nN, L; 
	double sum;
	string st="";
	for(L=nLayer-1; L>0;L--) //   
	{
		if (LayerType[L+1]==ltOut) //    out (Target)
		{
			for(N=0;N<nNeuron[L];N++) //     
			{
				Err[L][N][0]=Out[L][N]*(1-Out[L][N])*(Out[L+1][N]-Out[L][N]);
				st=st+" errOut "+DTS(Err[L][N][0]);
			}
		}
		if (LayerType[L+1]==ltBack) //      
		{
			for(N=0;N<nNeuron[L];N++) //     
			{ 
				sum=0;
				for(nN=0;nN<nNeuron[L+1];nN++) sum+=Err[L+1][nN][0]*W[L+1][nN][N];
				Err[L][N][0]=Out[L][N]*(1-Out[L][N])*sum;
				st=st+" err "+DTS(Err[L][N][0]);
			}
		}
	}
//	Comment(st);
}
//------------------------------------------------
void CNeuroNet::ChangeWeight() //  
{
	int pN, N, L, ea;
	double max=0;
	for(L=nLayer-1;L>0;L--) 
	{
		if (LayerType[L]==ltBack) //  -   
		{
			for(N=0;N<nNeuron[L];N++) //   
				if (MathAbs(Err[L][N][0])>MathAbs(max)) { max=Err[L][N][0]; ea=N; }
			//   
			for(N=0;N<nNeuron[L];N++)
			{
				for(pN=0;pN<nNeuron[L-1];pN++) 
				{
					//   ""  
					if (N==ea) dW[L][N][pN]=2*Speed*Err[L][N][0]*Out[L-1][pN]+Impuls*dW[L][N][pN];
					else dW[L][N][pN]=Speed*Err[L][N][0]*Out[L-1][pN]+Impuls*dW[L][N][pN];
					//   
					if (MathAbs(dW[L][N][pN])<1e-8) dW[L][N][pN]=1e-7*Sign(dW[L][N][pN]);
					W[L][N][pN]=W[L][N][pN]+dW[L][N][pN];
				}
				//   ""  
				if (N==ea) dThresh[L][N]=2*Speed*Err[L][N][0]+Impuls*dThresh[L][N];
				else dThresh[L][N]=Speed*Err[L][N][0]+Impuls*dThresh[L][N];
				//   
				if (MathAbs(dThresh[L][N])<1e-8) dThresh[L][N]=10*dThresh[L][N];
				Thresh[L][N]=Thresh[L][N]+dThresh[L][N];
			}
		}
	}
}
//------------------------------------------------ Step
void CNeuroNet::Step(int pat)
{
	int i;
	for(i=0;i<nNeuron[0];i++) Out[0][i]=in[i][pat]; //   
	for(i=0;i<nNeuron[nLayer];i++) Out[nLayer][i]=out[i][pat]; //    
	CalculateLayer(); //  
	CalculateError(); //  
	ChangeWeight(); //  
}
//------------------------------------------------ Walk
void CNeuroNet::Walk()
{
	int i, pat;
	for(pat=0;pat<nPattern;pat++) //      
	{
		for(i=0;i<nNeuron[0];i++) Out[0][i]=in[i][pat]; //   
		for(i=0;i<nNeuron[nLayer];i++) Out[nLayer][i]=out[i][pat]; //    
		CalculateLayer(); //  
		CalculateError(); //  
		ChangeWeight(); //  
	}
}
//------------------------------------------------ CountError
bool CNeuroNet::CountError()
{
	double dmax, ser2, err;
	int i, pat, ipat;
	bool bError=false;
	
	dmax=0; ser2=0;
	for(pat=0; pat<nPattern; pat++) //   
	{
		for(i=0;i<nNeuron[0];i++) Out[0][i]=in[i][pat]; //   
		for(i=0;i<nNeuron[nLayer];i++) Out[nLayer][i]=out[i][pat]; //    
		CalculateLayer(); //  
		//       ,   
		for(i=0;i<nNeuron[nLayer-1];i++) 
		{
			err=Out[nLayer][i]-Out[nLayer-1][i];	//  
			if (MathAbs(err)>Delta) bError=true;	//   
			if (MathAbs(err)>MathAbs(dmax))	{ ipat=pat; dmax=err; }// 
			ser2+=(err*err);// 
		}
	}
	sErr=ser2; dMaxErr=dmax; iMaxErr=ipat;
	return (bError);
}
//------------------------------------------------
bool CNeuroNet::Run()	//   
{
	int i, ipat;
	bool bError=true;
	double err, ser2, dmax;
	bInProc=true;
	loop=1;
	while(!bStop && (bError || (nCycle>0 && loop<nCycle))) //    
	{
		for(pat=0;pat<nPattern;pat++) //      
		{
			for(i=0;i<nNeuron[0];i++) Out[0][i]=in[i][pat]; //   
			for(i=0;i<nNeuron[nLayer];i++) Out[nLayer][i]=out[i][pat]; //    
			CalculateLayer(); //  
			CalculateError(); //  
			ChangeWeight(); //  
		}
		
		bError=CountError(); //   
		loop++;
	}
	bInProc=false;
	return (!bError);//    -    
}
//------------------------------------------------
void CNeuroNet::LinearCovariation()//    [0,1]
{
	int pat, N, pN, L, k=1;
	double max, min, p;
	/*
	//  
	for (N=0; N<nNeuron[0]; N++) //     
	{
		min=in[N][0]; //     
		for (pat=0; pat<nPattern; pat++) 
			if (in[N][pat]<min) 
				min=in[N][pat];
		//      
		for (pat=0; pat<nPattern; pat++)  in[N][pat]-=min;
		max=in[N][0]; //     
		for (pat=0; pat<nPattern; pat++)  if (in[N][pat]>max) max=in[N][pat];
		 //   [-1,1]
		if (max!=0) for (pat=0; pat<nPattern; pat++) in[N][pat]=2*(in[N][pat]/max)-1;
	}
/*
	for(pat=0;pat<nPattern;pat++)	//      
	{
		p=0;
		for(N=0;N<nNeuron[0];N++) p+=in[N][pat]*in[N][pat];
		p=sqrt(p);
		if (p!=0) for(N=0;N<nNeuron[0];N++) in[N][pat]=in[N][pat]/p;
	}
*/
	//  
	for (L=0; L<nLayer; L++) 
	{
		if (LayerType[L]==ltBack && nNeuron[L]>1)
		{
			for (pN=0; pN<nNeuron[L-1]; pN++) 
			{
				min=W[L][0][pN]; //     
				for (N=0; N<nNeuron[L]; N++) if (W[L][N][pN]<min) min=W[L][N][pN];
				//      
				for (N=0; N<nNeuron[L]; N++) W[L][N][pN]-=min;
				max=W[L][0][pN]; //     
				for (N=0; N<nNeuron[L]; N++) if (W[L][N][pN]>max) max=W[L][N][pN];
				 //   [-1,1]
				if (max!=0) for (N=0; N<nNeuron[L]; N++) W[L][N][pN]=2*(W[L][N][pN]/max)-1;
			}
		}
		if (LayerType[L]==ltBack && nNeuron[L]==1) //      
		{
			N=0;
			min=W[L][0][0]; //     
			for (pN=0; pN<nNeuron[L-1]; pN++) if (W[L][N][pN]<min) min=W[L][N][pN];
			//      
			for (pN=0; pN<nNeuron[L-1]; pN++) W[L][N][pN]-=min;
			max=W[L][N][0]; //     
			for (pN=0; pN<nNeuron[L-1]; pN++) if (W[L][N][pN]>max) max=W[L][N][pN];
			 //   [-1,1]
			if (max!=0) for (pN=0; pN<nNeuron[L-1]; pN++) W[L][N][pN]=2*(W[L][N][pN]/max)-1;
		}
	}
}
//------------------------------------------------
void CNeuroNet::Load(string FileName) //  /
{
	CFile file;
	int i, j, k;
	FileName= "F:\\ForexWork\\MetaTraders\\MetaTrader 4 Ft-Trade\\experts\\files\\MA25_15.in";
	file.Open(FileName, FILE_READ);
	//   
	nBAR=FileReadInteger(file.Handle());//  
	nIN=FileReadInteger(file.Handle()); //  
	nOUT=FileReadInteger(file.Handle());//  
	//    
	for (i=0; i<nBAR; i++) for (j=0; j<nIN; j++) in[j][i]=FileReadDouble(file.Handle()); 
	//    
	for (i=0; i<nBAR; i++) for (j=0; j<nOUT; j++) out[j][i]=FileReadDouble(file.Handle());
	file.Close(); //   
}
//------------------------------------------------
void CNeuroNet::LoadHistory(string FileName) //  /
{
	CFile file;
	int i, j, k;
	FileName= "F:\\ForexWork\\MetaTraders\\MetaTrader 4 Ft-Trade\\experts\\files\\MA25_15.bar";
	file.Open(FileName, FILE_READ);
	//    
	for (i=0; i<nBAR; i++) 
	{
		bar[i].open=FileReadDouble(file.Handle());
		bar[i].high=FileReadDouble(file.Handle());
		bar[i].low=FileReadDouble(file.Handle());
		bar[i].close=FileReadDouble(file.Handle());
	}
	file.Close(); //   
}
//------------------------------------------------
void CNeuroNet::SaveW(string FileName) //    
{
	int N, pN, L=1;
	CFile file;
	FileName= "F:\\ForexWork\\MetaTraders\\MetaTrader 4 Ft-Trade\\experts\\files\\NeuroWgh.dat";
	file.Open(FileName, FILE_WRITE);
	//  
	for(N=0;N<nNeuron[L];N++) 
	{
		for(pN=0;pN<nNeuron[L-1];pN++) 
			FileWrite(file.Handle(), W[L][N][pN], sizeof(double));
	}
	file.Flush(); file.Close();
}
//------------------------------------------------
bool CNeuroNet::TrainMPS()	//  
{
	int i, ipat;
	bool bError=true;
	double ser2, dmax=0;
	double TP=50;
	bInProc=true; bStop=false;
	loop=1;
	while(!bStop && (bError || (nCycle>0 && loop<nCycle))) //    
	{
		ser2=0;
		pat=0; //     
		for(i=0;i<nNeuron[0];i++) Out[0][i]=in[i][pat]; //   
		CalculateLayer(); //  
		 //    ,  ,  
		TradePos=TradeDir(Out[nLayer-1][0]); ipat=pat;
		
		for(pat=1;pat<nPattern;pat++) //      
		{
			for(i=0;i<nNeuron[0];i++) Out[0][i]=in[i][pat]; //   
			CalculateLayer(); //  
			//  /    [3]
			ProfitPos=1e4*TradePos*(bar[pat].close-bar[ipat].close);
			//       -
			if (/*TradeDir(Out[nLayer-1][0])!=TradePos || */ProfitPos>=TP || ProfitPos<=-TP) 
			{
				//  
				if (pat==48) CalculateLayer(); //  
				ser2+=ProfitPos;
				//    
				Out[nLayer][0]=Sigmoid(0.1*TradePos*ProfitPos);
				//   ,   
				for(i=0;i<nNeuron[0];i++) Out[0][i]=in[i][ipat]; 
				CalculateLayer(); //  
				CalculateError(); //  
				ChangeWeight(); //  
				if (pat==48) CalculateLayer(); //  
				//    
				for(i=0;i<nNeuron[0];i++) Out[0][i]=in[i][pat]; 
				CalculateLayer(); //  
				 //    ,  ,  
				TradePos=TradeDir(Out[nLayer-1][0]); 
				ipat=pat;
			}
		}

		sErr=ser2; dMaxErr=ser2; iMaxErr=ser2;
		loop++;
	}
	bInProc=false;
	return (!bError);//    -    
}

//------------------------------------------------	Sign
int Sign(double x) { if (x>=0) return (1); return (-1); } //   
//------------------------------------------------	Sign
double Sigma(double x) { if (x>0) return (1); if (x<0) return (0); return(0.5); } //   
//------------------------------------------------	Sigmoid
double Sigmoid(double x) { return(1/(1+exp(-x))); } //   
//------------------------------------------------	TradeDir
int TradeDir(double x) { if (x>=0.5) return(1); return(-1); } //  
//------------------------------------------------
/*
void ProcessedMsg()
{
	//   
	CFile file;
	int i, j, k;
	string FileName; //    
	FileName= "F:\\ForexWork\\MetaTraders\\MetaTrader 4 Ft-Trade\\experts\\files\\MA25_15.in";
	file.Open(FileName, FILE_READ);
	//   
	file.Read(nBAR, sizeof(int));//  
	file.Read(nIN, sizeof(int)); //  
	file.Read(nOUT, sizeof(int));//  
	//    
	for (i=0; i<nBAR; i++) for (j=0; j<nIN; j++) file.Read(&NN.in[j][i], sizeof(double)); 
	//    
	for (i=0; i<nBAR; i++) for (j=0; j<nOUT; j++) file.Read(&NN.out[j][i], sizeof(double));
	file.Close(); //   

	FileName= "F:\\ForexWork\\MetaTraders\\MetaTrader 4 Ft-Trade\\experts\\files\\MA25_15.bar";
	file.Open(FileName, CFile::modeRead);
	//    
	for (i=0; i<nBAR; i++)  for (j=0; j<4; j++) file.Read(&NN.bar[j][i], sizeof(double)); 
	file.Close(); //   

	NN.nNeuron[0]=nIN;	//    
	NN.nNeuron[1]=2*nIN;//     // 
	NN.nNeuron[2]=nIN;	//     
	NN.nNeuron[3]=5;	//     
	NN.nNeuron[4]=nOUT;	//      (   Target)
	NN.nNeuron[5]=nOUT;	//     
	NN.Init(1000, 4, 6000, 1e-8, 0.15, 0.15); //    

//	NN.TrainNetwork(1);	//     
	NN.TrainMPS();	//   

	//       MQL
	file.Open(FileName+".wgh", CFile::modeCreate); file.Close();
	file.Open(FileName+".wgh", CFile::modeWrite);
	//   
	file.Write(&nIN, sizeof(int)); //  
	file.Write(&nOUT, sizeof(int));//  
	file.Write(&NN.nLayer, sizeof(int));//  
	//      
	for (k=0; k<NN.nLayer; k++) file.Write(&NN.nNeuron[k], sizeof(int)); //  
	//      
	for (k=1; k<NN.nLayer; k++)
		for (i=0; i<NN.nNeuron[k]; i++)
		{
			for (j=0; j<NN.nNeuron[k-1]; j++)
				file.Write(&NN.W[k][i][j], sizeof(double));	//  
			file.Write(&NN.Thresh[k][i], sizeof(double));	//    
		}
	file.Flush(); file.Close();
}
*/

double ND(double d, int n=-1) { if (n<0) return NormalizeDouble(d, 4); return NormalizeDouble(d, n); }
string DTS(double d, int n=-1) { if (n<0) return DoubleToString(d); return DoubleToString(d, n); }
string ITS(int d, int n=-1) { return IntegerToString(d); }
string TTS(datetime d) { return TimeToString(d); }
